import logging
import typing

from django.contrib.auth.models import User
from django.core.exceptions import ObjectDoesNotExist
from django.template import Context, Template
from django.utils.module_loading import import_string

from notification.models import Message, MessageTemplate, Notification

logger = logging.getLogger(__name__)


class BaseNotificationBackend:
    id = None
    message_subtype = "plain"
    message_class = Message
    notification_class = Notification

    def __init__(self, fail_silently: bool = False, **kwargs) -> None:
        self.fail_silently = fail_silently

    def on_failure(self, message, recipient, exc, save=False, notify_kwargs=None):
        if not self.fail_silently:
            raise exc

        logger.error(
            "Failed to send message to recipient: %s, error: %s",
            recipient,
            exc,
        )
        if save:
            if message.pk is None:
                message.save()

            if recipient is None:
                return

            self.notification_class.objects.create(
                to=recipient,
                message=message,
                notify_kwargs=notify_kwargs or {},
                is_sent=False,
            )

    def on_success(self, message, recipient, save=False, notify_kwargs=None):
        logger.info("Successfully send message to recipient: %s", recipient)
        if save:
            if message.pk is None:
                message.save()

            if recipient is None:
                return

            self.notification_class.objects.create(
                to=recipient,
                message=message,
                notify_kwargs=notify_kwargs or {},
                is_sent=True,
            )

    def render_template(self, template: MessageTemplate, context: dict) -> str:
        try:
            return Template(template.content.plain).render(Context(context))
        except Exception as e:
            raise ValueError("Render message failed: %s" % e)

    def make_content(
        self, title, content, recipients, recipient_field, **kwargs
    ) -> dict:
        raise NotImplementedError

    def get_recipient(
        self,
        recipient: list[User],
        recipient_field: typing.Union[str, typing.Callable],
    ) -> typing.Iterable[str]:
        if callable(recipient_field):
            return recipient_field(recipient)
        else:
            return getattr(recipient, recipient_field)

    def perform_send(self, message, recipients, recipient_field, save, **kwargs) -> None:
        raise NotImplementedError

    def clear(self) -> None:
        """
        Clear notification
        """
        self.notification_class.objects.filter(message__msg_type=self.id).delete()
        self.message_class.objects.filter(msg_type=self.id).delete()

    def send_with_template(
        self,
        recipients: list[User],
        template: MessageTemplate,
        context: dict,
        title: str = None,
        mark: str = None,
        save: bool = False,
        recipient_field: typing.Union[str, typing.Callable] = None,
        message_kwargs: dict = None,
        **kwargs,
    ) -> None:
        """
        Send notification to receivers with template
        """
        rendered_content = self.render_template(template, context)
        message_kwargs = message_kwargs or {}
        if template.message_kwargs:
            message_kwargs = template.message_kwargs.update(message_kwargs)

        title = title or template.title
        message_content = self.make_content(
            title, rendered_content, recipients, recipient_field, **message_kwargs
        )
        message = self.message_class(
            title=title,
            content=message_content,
            template=template,
            render_kwargs=context,
            mark=mark,
            msg_type=self.id,
        )
        self.perform_send(message, recipients, recipient_field, save=save, **kwargs)

    def send(
        self,
        title: str,
        recipients: list[User],
        content: str,
        mark: str = None,
        save: bool = False,
        recipient_field: typing.Union[str, typing.Callable] = None,
        message_kwargs: dict = None,
        **kwargs,
    ) -> None:
        """
        Send notification to receivers
        """

        message_content = self.make_content(
            title, content, recipients, recipient_field, **(message_kwargs or {})
        )
        message = self.message_class(
            title=title, content=message_content, mark=mark, msg_type=self.id
        )
        self.perform_send(message, recipients, recipient_field, save=save, **kwargs)


def notify(
    recipients: typing.Optional[list[User]],
    title: str = None,
    message: str = None,
    template_code: str = None,
    context: dict = None,
    save: bool = False,
    backends: typing.Tuple[typing.Union[BaseNotificationBackend, str]] = None,
    recipient_field: typing.Union[str, typing.Callable] = None,
    message_kwargs: dict = None,
    mark: dict = None,
    **kwargs,
):
    if message is None and not message_kwargs and not all([template_code, context]):
        raise ValueError("You must provide message or template or message_kwargs.")

    if not backends:
        raise ValueError("You must provide at least one backend.")

    if not recipients and save:
        logger.warning("No recipients provided, `save=True` will be ignored.")

    for backend_cls in backends:
        if isinstance(backend_cls, str):
            backend_cls = import_string(backend_cls)

        if template_code:
            try:
                template = MessageTemplate.objects.get(code=template_code)
            except ObjectDoesNotExist:
                raise ValueError(f"Template: {template_code} doesn't exist.")
            # `kwargs` is prior to `kwargs` defined in template
            if template.backend_kwargs and isinstance(template.backend_kwargs, dict):
                backend_kwargs = template.backend_kwargs
                backend_kwargs.update(kwargs)
            else:
                backend_kwargs = kwargs

            backend = backend_cls(**backend_kwargs)
            backend.send_with_template(
                recipients,
                template,
                context,
                title=title,
                save=save,
                recipient_field=recipient_field,
                message_kwargs=message_kwargs,
                mark=mark,
            )
        else:
            backend = backend_cls(**kwargs)
            backend.send(
                title,
                recipients,
                message,
                save=save,
                recipient_field=recipient_field,
                message_kwargs=message_kwargs,
                mark=mark
            )



import typing
from email.mime.base import MIMEBase

from django.contrib.auth.models import User
from django.core.mail import EmailMessage
from django.template import Context, Template

from notification.base import BaseNotificationBackend, notify
from notification.models import Message, MessageTemplate


class EmailNotificationBackend(BaseNotificationBackend):
    """
    A backend handle email message
    """

    id = "email"
    message_subtype = "html"

    def render_template(self, template: MessageTemplate, context: dict) -> str:
        try:
            return Template(template.content.html).render(Context(context))
        except Exception as e:
            raise ValueError("Render message failed: %s" % e)

    def make_content(self, title, content, recipients, recipient_field, **kwargs):
        return {
            "subject": title,
            "body": content,
            **kwargs,
        }

    def perform_send(
        self, message: Message, recipients, recipient_field, save, **kwargs
    ):
        """
        For details, see: ...
        """
        email_content = message.content
        for recipient in recipients:
            recipient_email = self.get_recipient(recipient, recipient_field)
            notify_kwargs = {"to": [recipient_email]}
            email = EmailMessage(**notify_kwargs, **email_content)
            email.content_subtype = self.message_subtype
            try:
                email.send()
            except Exception as e:
                self.on_failure(message, recipient, e, save, notify_kwargs)
            else:
                self.on_success(message, recipient, save, notify_kwargs)


def notify_by_email(
    recipients: list[User],
    email_field: typing.Union[str, typing.Callable] = "email",
    title: str = None,
    message: str = None,
    context: dict = None,
    template_code: str = None,
    cc: list[str] = None,
    attachments: list[MIMEBase] = None,
    save=False,
    **kwargs,
):
    """
    Shortcut for email notification
    """
    message_kwargs = {
        "cc": cc,
        "attachments": attachments,
    }

    return notify(
        recipients,
        title=title,
        message=message,
        context=context,
        template_code=template_code,
        backends=(EmailNotificationBackend,),
        save=save,
        recipient_field=email_field,
        message_kwargs=message_kwargs,
        **kwargs,
    )



from django.contrib.auth.models import User

from notification.base import BaseNotificationBackend, notify
from notification.models import Message


class DummyNotificationBackend(BaseNotificationBackend):
    """
    A backend handle dummy message.
    """

    id = "dummy"

    def make_content(self, title, content, recipients, recipient_field, **kwargs):
        return content

    def perform_send(
        self, message: Message, recipients, recipient_field, save, **kwargs
    ) -> None:
        """
        Send dummy message
        """
        for recipient in recipients:
            self.on_success(message, recipient, save)


def notify_by_dummy(
    recipients: list[User],
    title: str = None,
    message: str = None,
    context: dict = None,
    template_code: str = None,
    save: bool = False,
    mark: dict = None,
    **kwargs,
):
    """
    Shortcut for dummy notification
    """
    return notify(
        recipients,
        title=title,
        message=message,
        context=context,
        template_code=template_code,
        backends=(DummyNotificationBackend,),
        save=save,
        message_kwargs={},
        mark=mark,
        **kwargs,
    )



import typing

import requests
from django.conf import settings
from django.contrib.auth.models import User
from django.core.exceptions import ImproperlyConfigured
from django.template import Context, Template
from markdownify import markdownify

from notification.base import BaseNotificationBackend, notify
from notification.models import Message, MessageTemplate


class DingTalkWorkMessageNotificationBackend(BaseNotificationBackend):
    """
    A backend handle dingtalk work message.
    For details, see: https://open.dingtalk.com/document/isvapp-server/asynchronous-sending-of-enterprise-session-messages
    """  # noqa

    id = "dingtalkworkmessage"
    message_subtype = "markdown"

    def __init__(self, *args, agent_id=None, app_key=None, app_secret=None, **kwargs):
        try:
            self.notification_settting = settings.DJANGO_USER_NOTIFICATION[self.id]
        except (AttributeError, KeyError):
            raise ImproperlyConfigured(
                "'DJANGO_USER_NOTIFICATION[{}]' must be "
                "set in settings.py".format(self.id)
            )

        try:
            self.agent_id = agent_id or self.notification_settting["agent_id"]
        except KeyError:
            raise ImproperlyConfigured(
                "'agent_id' must be set in "
                "settings.DJANGO_USER_NOTIFICATION[{}]".format(self.id)
            )

        try:
            self.app_key = app_key or self.notification_settting["app_key"]
        except KeyError:
            raise ImproperlyConfigured(
                "'app_key' must be set in "
                "settings.DJANGO_USER_NOTIFICATION[{}]".format(self.id)
            )

        try:
            self.app_secret = app_secret or self.notification_settting["app_secret"]
        except KeyError:
            raise ImproperlyConfigured(
                "'app_secret' must be set in "
                "settings.DJANGO_USER_NOTIFICATION[{}]".format(self.id)
            )

        super().__init__(*args, **kwargs)

    def render_template(self, template: MessageTemplate, context: dict) -> str:
        try:
            html_content = Template(template.content.html).render(Context(context))
            return markdownify(html_content)
        except Exception as e:
            raise ValueError("Render message failed: %s" % e)

    def make_content(
        self, title, content, recipients, recipient_field, **kwargs
    ) -> dict:
        return {
            "msgtype": self.message_subtype,
            self.msgtype: {"title": title, "text": content},
        }

    def get_access_token(self):
        url = "https://oapi.dingtalk.com/gettoken"
        params = {
            "appkey": self.app_key,
            "appsecret": self.app_secret,
        }
        resp = requests.get(url, params=params)
        ret = resp.json()
        return ret["access_token"]

    def perform_send(
        self, message: Message, recipients, recipient_field, save, **kwargs
    ) -> None:
        """
        Send dingtalk work message
        """
        url = "https://oapi.dingtalk.com/topapi/message/corpconversation/asyncsend_v2"
        params = {"access_token": self.get_access_token()}
        for recipient in recipients:
            dingtalk_userid = self.get_recipient(recipient, recipient_field)
            notify_kwargs = {"userid_list": dingtalk_userid}
            json = {
                "agent_id": self.agent_id,
                "msg": message.content,
                **notify_kwargs,
                **kwargs,
            }
            try:
                resp = requests.post(url, params=params, json=json)
                resp.raise_for_status()
                ret = resp.json()
                assert ret["errcode"] == 0, ret["errmsg"]
            except Exception as e:
                self.on_failure(message, recipient, e, save, notify_kwargs)
            else:
                self.on_success(message, recipient, save, notify_kwargs)


def notify_by_dingtalk_workmessage(
    recipients: list[User],
    userid_field: typing.Union[str, typing.Callable],
    title: str = None,
    message: str = None,
    context: dict = None,
    template_code: str = None,
    save: bool = False,
    **kwargs,
):
    """
    Shortcut for dingtalk work message notification
    """
    return notify(
        recipients,
        title=title,
        message=message,
        context=context,
        template_code=template_code,
        backends=(DingTalkWorkMessageNotificationBackend,),
        save=save,
        recipient_field=userid_field,
        message_kwargs={},
        **kwargs,
    )



# Generated by Django 3.2.12 on 2022-07-29 07:54

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import django_quill.fields


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Message',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('title', models.CharField(blank=True, max_length=64, null=True, verbose_name='Title')),
                ('mark', models.CharField(blank=True, db_index=True, max_length=64, null=True, verbose_name='Message Mark')),
                ('msg_type', models.CharField(db_index=True, max_length=64, verbose_name='Message Type')),
                ('content', models.TextField(blank=True, null=True, verbose_name='Content')),
                ('render_kwargs', models.JSONField(blank=True, default=dict, verbose_name='Render Kwargs')),
                ('created_at', models.DateTimeField(auto_now_add=True, verbose_name='Created At')),
            ],
            options={
                'verbose_name': 'Message',
                'db_table': 'message',
            },
        ),
        migrations.CreateModel(
            name='MessageTemplate',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=64, verbose_name='Template Name')),
                ('description', models.TextField(blank=True, null=True, verbose_name='Description')),
                ('code', models.CharField(max_length=4, unique=True, verbose_name='Template Code')),
                ('title', models.CharField(blank=True, max_length=64, null=True, verbose_name='Message Title')),
                ('content', django_quill.fields.QuillField(verbose_name='Template Content')),
                ('backend_kwargs', models.JSONField(blank=True, default=dict, verbose_name='Backend Kwargs')),
                ('message_kwargs', models.JSONField(blank=True, default=dict, verbose_name='Message Kwargs')),
            ],
            options={
                'verbose_name': 'Message Template',
                'db_table': 'message_template',
            },
        ),
        migrations.CreateModel(
            name='Notification',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('has_read', models.BooleanField(default=False, verbose_name='Read Or Not')),
                ('is_ignored', models.BooleanField(default=False, verbose_name='Ignored Or Not')),
                ('is_sent', models.BooleanField(default=False, verbose_name='Sent Or Not')),
                ('notify_kwargs', models.JSONField(blank=True, default=dict, verbose_name='Notify Kwargs')),
                ('created_at', models.DateTimeField(auto_now_add=True, verbose_name='Created At')),
                ('updated_at', models.DateTimeField(auto_now=True, verbose_name='Updated At')),
                ('message', models.ForeignKey(db_constraint=False, on_delete=django.db.models.deletion.CASCADE, related_name='notifications', to='notification.message', verbose_name='Message')),
                ('to', models.ForeignKey(db_constraint=False, on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL, verbose_name='Receiver')),
            ],
            options={
                'verbose_name': 'Notification',
                'db_table': 'notification',
            },
        ),
        migrations.AddField(
            model_name='message',
            name='template',
            field=models.ForeignKey(blank=True, db_constraint=False, null=True, on_delete=django.db.models.deletion.CASCADE, to='notification.messagetemplate', verbose_name='Template'),
        ),
    ]



from asgiref.sync import async_to_sync
from django.contrib.auth.models import User

from notification.base import BaseNotificationBackend, notify
from notification.models import Message
from notification.utils import get_group_name

try:
    import channels.layers
except ImportError:
    pass


class WebsocketNotificationBackend(BaseNotificationBackend):
    """
    A backend handle websocket message.

    """

    id = "websocket"
    message_subtype = "plain"

    def make_content(
        self,
        title: str,
        content: str,
        recipients,
        recipient_field,
        msgtype="notify",
        **kwargs,
    ):
        return {
            "type": f"{msgtype}.message",
            "msgtype": msgtype,
            "title": title,
            "message": content,
            **kwargs,
        }

    def perform_send(
        self,
        message: Message,
        recipients: list[User],
        recipient_field,
        save,
        **kwargs,
    ):
        """
        For details, see: https://ding-doc.dingtalk.com/doc#/serverapi2/qf2nxq
        """
        channel_layer = channels.layers.get_channel_layer()
        for recipient in recipients:
            notify_kwargs = {"group_name": get_group_name(recipient.pk)}
            try:
                async_to_sync(channel_layer.group_send)(
                    notify_kwargs["group_name"], message.content
                )
            except Exception as e:
                self.on_failure(message, recipient, e, save, notify_kwargs)
            else:
                self.on_success(message, recipient, save, notify_kwargs)


def notify_by_websocket(
    recipients: list[User],
    title: str = None,
    message: str = None,
    context: dict = None,
    template_code: str = None,
    msgtype="notify",
    save: bool = False,
    **kwargs,
):
    """
    Shortcut for websocket notification
    """
    message_kwargs = {"msgtype": msgtype}

    return notify(
        recipients,
        title=title,
        message=message,
        context=context,
        template_code=template_code,
        backends=(WebsocketNotificationBackend,),
        save=save,
        message_kwargs=message_kwargs,
        **kwargs,
    )



import json
import typing

from alibabacloud_dysmsapi20170525.client import Client as Dysmsapi20170525Client
from alibabacloud_dysmsapi20170525.models import SendSmsRequest
from alibabacloud_tea_openapi import models as open_api_models
from django.conf import settings
from django.contrib.auth.models import User
from django.core.exceptions import ImproperlyConfigured

from notification.base import BaseNotificationBackend, notify


class AliyunSMSNotificationBackend(BaseNotificationBackend):
    """
    A backend handle aliyun sms message.
    For details, see: https://help.aliyun.com/document_detail/419273.htm?spm=a2c4g.11186623.0.0.34375695hoVPSt
    """  # noqa

    id = "aliyunsms"
    message_subtype = "plain"
    endpoint = "dysmsapi.aliyuncs.com"

    def __init__(
        self, *args, access_key_id=None, access_key_secret=None, sign_name=None, **kwargs
    ):
        try:
            self.notification_settting = settings.DJANGO_USER_NOTIFICATION[self.id]
        except (AttributeError, KeyError):
            raise ImproperlyConfigured(
                "'DJANGO_USER_NOTIFICATION[{}]' must be set in settings.py".format(
                    self.id
                )
            )

        try:
            access_key_id = access_key_id or self.notification_settting["access_key_id"]
        except KeyError:
            raise ImproperlyConfigured(
                "'access_key_id' must be set "
                "in settings.DJANGO_USER_NOTIFICATION[{}]".format(self.id)
            )

        try:
            access_key_secret = (
                access_key_secret or self.notification_settting["access_key_secret"]
            )
        except KeyError:
            raise ImproperlyConfigured(
                "'access_key_secret' must be set "
                "in settings.DJANGO_USER_NOTIFICATION[{}]".format(self.id)
            )

        try:
            self.sign_name = sign_name or self.notification_settting["sign_name"]
        except KeyError:
            raise ImproperlyConfigured(
                "'sign_name' must be set "
                "in settings.DJANGO_USER_NOTIFICATION[{}]".format(self.id)
            )

        config = open_api_models.Config(
            access_key_id=access_key_id,
            access_key_secret=access_key_secret,
            endpoint=self.endpoint,
        )
        self.client = Dysmsapi20170525Client(config)
        super().__init__(*args, **kwargs)

    def make_content(self, title, content, receivers, recipient_field, **kwargs) -> dict:
        return {
            "sign_name": kwargs.get("sign_name") or self.sign_name,
            "template_code": kwargs["template_code"],
            "template_param": json.dumps(kwargs["template_param"]),
        }

    def perform_send(self, message, recipients, recipient_field, save, **kwargs):
        """
        For details, see: https://help.aliyun.com/document_detail/419273.htm?spm=a2c4g.11186623.0.0.34375695hoVPSt
        """  # noqa
        for recipient in recipients:
            phone = self.get_recipient(recipient, recipient_field)
            notify_kwargs = {"phone_numbers": str(phone)}
            try:
                request = SendSmsRequest(**notify_kwargs, **message.content)
                self.client.send_sms(request)
            except Exception as e:
                self.on_failure(message, recipient, e, save, notify_kwargs)
            else:
                self.on_success(message, recipient, save, notify_kwargs)


def notify_by_aliyun_sms(
    recipients: list[User],
    phone_field: typing.Union[str, typing.Callable],
    template_code: str,
    context: dict,
    sign_name: str = None,
    save=False,
    **kwargs,
):
    """
    Shortcut for aliyun sms message notification
    """
    message_kwargs = {
        "template_code": template_code,
        "template_param": context,
        "sign_name": sign_name,
    }

    return notify(
        recipients,
        backends=(AliyunSMSNotificationBackend,),
        save=save,
        recipient_field=phone_field,
        message_kwargs=message_kwargs,
        **kwargs,
    )



from base64 import b64encode


def make_etag(*args):
    """
    Generate custom etag
    """
    return "-".join(b64encode(tag.encode("utf-8")).decode("utf-8") for tag in args)


def get_group_name(user_id: int):
    return f"notify-{user_id}"


def get_notification_backend_class(msg_type):
    """
    Get notification backend.
    """
    from notification import backends
    from notification.base import BaseNotificationBackend

    for att in dir(backends):
        attr_value = (att, getattr(backends, att))
        if (
            attr_value is issubclass(BaseNotificationBackend, attr_value)
            and getattr(attr_value, "id") == msg_type
        ):
            return attr_value
    else:
        raise ValueError(f"Notification backend {msg_type} doesn't exist.")



import typing

import requests
from django.conf import settings
from django.contrib.auth.models import User
from django.core.exceptions import ImproperlyConfigured

from notification.base import BaseNotificationBackend, notify
from notification.models import Message


class DingTalkToDoTaskNotificationBackend(BaseNotificationBackend):
    """
    A backend handle dingtalk todo message.
    https://open.dingtalk.com/document/orgapp-server/add-dingtalk-to-do-task
    """

    id = "dingtalktodotask"
    message_subtype = "plain"

    def __init__(self, *args, app_key=None, app_secret=None, **kwargs):
        try:
            self.notification_settting = settings.DJANGO_USER_NOTIFICATION[self.id]
        except (AttributeError, KeyError):
            raise ImproperlyConfigured(
                "'DJANGO_USER_NOTIFICATION[{}]' must be "
                "set in settings.py".format(self.id)
            )

        try:
            self.app_key = app_key or self.notification_settting["app_key"]
        except KeyError:
            raise ImproperlyConfigured(
                "'app_key' must be set in "
                "settings.DJANGO_USER_NOTIFICATION[{}]".format(self.id)
            )

        try:
            self.app_secret = app_secret or self.notification_settting["app_secret"]
        except KeyError:
            raise ImproperlyConfigured(
                "'app_secret' must be set in "
                "settings.DJANGO_USER_NOTIFICATION[{}]".format(self.id)
            )

        super().__init__(*args, **kwargs)

    def make_content(
        self, title, content, recipients, recipient_field, **kwargs
    ) -> dict:
        return {"subject": title, "description": content, **kwargs}

    def get_access_token(self):
        url = "https://oapi.dingtalk.com/gettoken"
        params = {
            "appkey": self.app_key,
            "appsecret": self.app_secret,
        }
        resp = requests.get(url, params=params)
        ret = resp.json()
        return ret["access_token"]

    def perform_send(
        self, message: Message, recipients, recipient_field, save, **kwargs
    ) -> None:
        """
        Send dingtalk work message
        For details, see: https://open.dingtalk.com/document/isvapp-server/asynchronous-sending-of-enterprise-session-messages
        """  # noqa
        url = "https://api.dingtalk.com/v1.0/todo/users/{unionid}/tasks"
        headers = {"x-acs-dingtalk-access-token": self.get_access_token()}
        for recipient in recipients:
            dingtalk_unionid = self.get_recipient(recipient, recipient_field)
            notify_kwargs = {
                "unionid": dingtalk_unionid,
                "executorIds": [dingtalk_unionid],
            }
            url = url.format(unionid=dingtalk_unionid)
            json = {"executorIds": [dingtalk_unionid], **message.content}
            try:
                resp = requests.post(url, headers=headers, json=json)
                ret = resp.json()
                assert resp.status_code == 200, ret["message"]
            except Exception as e:
                self.on_failure(message, recipient, e, save, notify_kwargs)
            else:
                self.on_success(message, recipient, save, notify_kwargs)


def notify_by_dingtalk_todotask(
    recipients: list[User],
    unioinid_field: typing.Union[str, typing.Callable],
    title: str = None,
    message: str = None,
    context: dict = None,
    template_code: str = None,
    participant_ids: list[str] = None,
    due_time: int = None,
    priority: int = None,
    save: bool = False,
    **kwargs,
):
    """
    Shortcut for dingtalk todo tasks notification
    """
    message_kwargs = {
        "participantIds": participant_ids,
        "dueTime": due_time,
        "priority": priority,
        "notifyConfigs": {"dingNotify": "1"},
    }

    return notify(
        recipients,
        title=title,
        message=message,
        context=context,
        template_code=template_code,
        backends=(DingTalkToDoTaskNotificationBackend,),  # noqa
        save=save,
        recipient_field=unioinid_field,
        message_kwargs=message_kwargs,
        **kwargs,
    )



from django.contrib import admin, messages
from django.contrib.admin import display
from django.db import models
from django.forms import Textarea
from django.utils import timezone
from django.utils.translation import gettext_lazy as _

from .models import Message, MessageTemplate, Notification

# Register your models here.


@admin.register(MessageTemplate)
class TemplateAdmin(admin.ModelAdmin):
    list_per_page = 15
    list_display = ("name", "code", "title", "description")
    list_filter = ("code",)
    search_fields = ["code", "name"]
    fields = [
        "name",
        "code",
        "title",
        "description",
        "backend_kwargs",
        "message_kwargs",
        "content",
    ]
    ordering = ("-id",)

    formfield_overrides = {
        models.TextField: {"widget": Textarea(attrs={"rows": 3, "cols": 38})},
        models.JSONField: {"widget": Textarea(attrs={"rows": 2, "cols": 38})},
    }


@admin.register(Notification)
class NotificationAdmin(admin.ModelAdmin):
    list_per_page = 15
    list_display = [
        "to",
        "get_message_title",
        "has_read",
        "is_ignored",
        "is_sent",
        "updated_at",
    ]
    list_select_related = ["to", "message"]
    list_filter = ["to", "is_sent", "message__msg_type"]
    search_fields = ["message__mark"]
    autocomplete_fields = ["to", "message"]
    actions = ["send", "read", "ignore"]
    ordering = ("-id",)

    def has_change_permission(self, request, obj=None):
        return False

    def send(self, request, queryset):
        return NotImplemented

    def read(self, request, queryset):
        if queryset.count() > 0:
            queryset.update(has_read=True, updated_at=timezone.now())

        self.message_user(request, "Notification read", level=messages.SUCCESS)

    def ignore(self, request, queryset):
        if queryset.count() > 0:
            queryset.update(is_ignored=True, updated_at=timezone.now())

        self.message_user(request, "Notification ignored", level=messages.SUCCESS)

    @display(description=_("Template"))
    def get_message_template(self, obj):
        return obj.message.template

    @display(description=_("Render kwargs"))
    def get_message_render_kwargs(self, obj):
        return obj.message.render_kwargs

    @display(description=_("Title"))
    def get_message_title(self, obj):
        return obj.message.title

    @display(description=_("Message Mark"))
    def get_message_mark(self, obj):
        return obj.message.mark

    @display(description=_("Message type"))
    def get_message_type(self, obj):
        return obj.message.msg_type

    @display(description=_("Content"))
    def get_message_content(self, obj):
        return obj.message.content

    def get_fieldsets(self, request, obj=None):
        if obj is None:
            return [(None, {"fields": ["to", "message"]})]

        return [
            (
                "Message",
                {
                    "fields": [
                        "get_message_title",
                        "get_message_type",
                        "get_message_mark",
                        "get_message_content",
                        "get_message_template",
                        "get_message_render_kwargs",
                    ]
                },
            ),
            (
                "Notification",
                {
                    "fields": [
                        "to",
                        "is_sent",
                        "has_read",
                        "is_ignored",
                        "notify_kwargs",
                        "created_at",
                        "updated_at",
                    ]
                },
            ),
        ]


@admin.register(Message)
class MessageAdmin(admin.ModelAdmin):
    list_per_page = 15
    list_display = [
        "title",
        "msg_type",
        "template",
        "mark",
        "created_at",
    ]
    list_filter = ("msg_type", "template")
    search_fields = ["mark", "title"]
    ordering = ("-id",)

    def has_change_permission(self, request, obj=None):
        return False



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['notification', 'notification.backends', 'notification.migrations']

package_data = \
{'': ['*'], 'notification': ['static/css/*']}

install_requires = \
['django-quill-editor>=0.1.40',
 'django>=3.1',
 'markdownify>=0.11.2',
 'requests>=2.27.1,<3.0.0']

extras_require = \
{'aliyunsms': ['alibabacloud-dysmsapi20170525>=2.0.16'],
 'channels': ['channels>=3.0.4']}

setup_kwargs = {
    'name': 'django-user-notification',
    'version': '0.7.10',
    'description': 'Django message notification package',
    'long_description': '# Django user notification\n\n[![Build Status](https://img.shields.io/github/workflow/status/anyidea/django-user-notification/CI/master)](https://github.com/anyidea/django-user-notification/actions?query=workflow%3ACI)\n[![GitHub license](https://img.shields.io/github/license/anyidea/django-user-notification)](https://github.com/anyidea/django-user-notification/blob/master/LICENSE)\n[![Documentation Status](https://readthedocs.org/projects/django-user-notification/badge/?version=latest)](https://django-user-notification.readthedocs.io/en/latest/?badge=latest)\n[![pypi-version](https://img.shields.io/pypi/v/django-user-notification.svg)](https://pypi.python.org/pypi/django-user-notification)\n![PyPI - Python Version](https://img.shields.io/pypi/pyversions/django-user-notification)\n[![PyPI - Django Version](https://img.shields.io/badge/django-%3E%3D3.1-44B78B)](https://www.djangoproject.com/)\n[![Code style: black](https://img.shields.io/badge/code%20style-black-000000.svg)](https://github.com/psf/black)\n\nOverview\n-----\nDjango user notification is intended to provide a way to send multiple types of notification messages to django users out of box.\n\nDocumentation\n-----\non the way...\n\nRequirements\n-----\n\n* Python 3.8, 3.9, 3.10\n* Django 3.1, 3.2, 4.0, 4.1\n\nInstallation\n-----\n\nInstall using `pip`...\n\n    pip install django-user-notification\n\nAdd `\'django_quill\'` and `\'notification\'` to your `INSTALLED_APPS` setting.\n```python\nINSTALLED_APPS = [\n    \'django.contrib.admin\',\n    ...\n    \'django_quill\',\n    \'notification\',\n]\n```\n\nQuick Start\n-----\n\nLet\'s take a look at a quick start of using Django user notification to send notification messages to users.\n\nRun the `notification` migrations using:\n\n    python manage.py migrate notification\n\n\nAdd the following to your `settings.py` module:\n\n```python\nINSTALLED_APPS = [\n    ...  # Make sure to include the default installed apps here.\n    \'django_quill\',\n    \'notification\',\n]\n\nDJANGO_USER_NOTIFICATION = {\n    "aliyunsms": {\n        "access_key_id": "Your Access Key ID",\n        "access_key_secret": "Your Access Key Secret",\n        "sign_name": "Your Sign Name",\n    },\n    "dingtalkchatbot": {\n        "webhook": "Your Webhook URL",\n    },\n    "dingtalkworkmessage": {\n        "agent_id": "Your App Agent ID",\n        "app_key": "Your App Key",\n        "app_secret": "Your App Secret",\n    },\n    "dingtalktodotask": {\n        "app_key": "Your App Key",\n        "app_secret": "Your App Secret",\n    },\n}\n```\n\nLet\'s send a notification\n\n``` {.python}\nfrom django.contrib.auth import get_user_model\nfrom notification.backends import notify_by_email, notify_by_dingtalk_workmessage\n\nUser = get_user_model()\n\nrecipient = User.objects.first()\n\n# send a dingtalk work message notification\nnotify_by_dingtalk_workmessage([recipient], phone_field="phone", title="This is a title", message="A test message")\n\n\n# send a email notiofication\nnotify_by_email([recipient], title="This is a title", message="A test message")\n```\n\n\nSend Message With Template\n--------------\n\n`django-user-notification` support send notifications with custom template, To\nspecify a custom message template you can provide the `template_code`\nand `context` parameters.\n\n1)  Create a template message with code named `TMP01` on django admin\n\n2)  Provide the `template_code` and `context` to `send` method:\n``` {.python}\n...\n\nnotify_by_email([recipient], template_code="TMP01", context={"content": "Hello"})\n```\n\nSupported backends\n-----------------------------\n\n- `DummyNotificationBackend`: send dummy message\n- `EmailNotificationBackend`: send email notification.\n- `WebsocketNotificationBackend`: send webdocket notification, need install extension: `channels`.\n- `AliyunSMSNotificationBackend`: send aliyun sms notification, need install extension: `aliyunsms`.\n- `DingTalkChatbotNotificationBackend`: send dingtalk chatbot notification.\n- `DingTalkToDoTaskNotificationBackend`: send dingtalk todo tasks notification\n- `DingTalkWorkMessageNotificationBackend`: send dingtalk work message notification.\n- `WechatNotificationBackend`: planning...\n\nRunning the tests\n-----------------\n\nTo run the tests against the current environment:\n\n``` {.bash}\n$ pytest tests/\n```\n\nChangelog\n---------\n\n### 0.7.0\n\n-   Initial release\n\nContributing\n------------\nAs an open source project, we welcome contributions. The code lives on [GitHub](https://github.com/anyidea/django-user-notification/)\n\n## Thanks\n\n[![PyCharm](docs/pycharm.svg)](https://www.jetbrains.com/?from=django-user-notification)\n',
    'author': 'Aiden Lu',
    'author_email': 'allaher@icloud.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/aiden520/django-user-notification',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'python_requires': '>=3.8',
}


setup(**setup_kwargs)



from django.conf import settings
from django.contrib.postgres.indexes import GinIndex
from django.db import models
from django.utils.translation import gettext_lazy as _
from django_quill.fields import QuillField


class NotificationQuerySet(models.QuerySet):
    def read(self):
        """
        return read notification messages
        """
        return self.filter(has_read=True)

    def unread(self, user):
        """
        return unread notification messages
        """
        return self.filter(has_read=False)

    def ignored(self):
        """
        Return ignored notification messages
        """
        return self.filter(is_ignored=True)

    def sent(self):
        """
        Return sent notification messages
        """
        return self.filter(is_sent=True)

    def unsent(self):
        """
        Return unsent notification messages
        """
        return self.filter(is_sent=False)


class Notification(models.Model):
    has_read = models.BooleanField(verbose_name=_("Read Or Not"), default=False)
    is_ignored = models.BooleanField(verbose_name=_("Ignored Or Not"), default=False)
    is_sent = models.BooleanField(verbose_name=_("Sent Or Not"), default=False)
    to = models.ForeignKey(
        settings.AUTH_USER_MODEL,
        verbose_name=_("Receiver"),
        on_delete=models.CASCADE,
        db_constraint=False,
    )
    message = models.ForeignKey(
        "Message",
        verbose_name=_("Message"),
        on_delete=models.CASCADE,
        db_constraint=False,
        related_name="notifications",
    )
    notify_kwargs = models.JSONField(
        verbose_name=_("Notify Kwargs"), blank=True, default=dict
    )
    created_at = models.DateTimeField(auto_now_add=True, verbose_name=_("Created At"))
    updated_at = models.DateTimeField(auto_now=True, verbose_name=_("Updated At"))

    objects = NotificationQuerySet.as_manager()

    class Meta:
        verbose_name = _("Notification")
        db_table = "notification"

    def __str__(self):
        return str(self.message)

    def mark_as_read(self):
        """
        Mark a notification message as read.
        """
        self.has_read = True
        self.save(update_fields=["has_read", "updated_at"])

    def mark_as_unread(self):
        """
        Mark a notification message as unread.
        """
        self.has_read = False
        self.save(update_fields=["has_read", "updated_at"])

    def mark_as_sent(self):
        """
        Mark a notification message as sent.
        """
        self.is_sent = True
        self.save(update_fields=["is_sent", "updated_at"])

    def mark_as_ignored(self):
        """
        Mark a notification message as ignored.
        """
        self.is_ignored = True
        self.save(update_fields=["is_ignored", "updated_at"])


class MessageTemplate(models.Model):
    """
    Message template
    """

    name = models.CharField(max_length=64, verbose_name=_("Template Name"))
    description = models.TextField(verbose_name=_("Description"), null=True, blank=True)
    code = models.CharField(max_length=4, verbose_name=_("Template Code"), unique=True)
    title = models.CharField(
        max_length=64, verbose_name=_("Message Title"), null=True, blank=True
    )
    content = QuillField(verbose_name=_("Template Content"))
    backend_kwargs = models.JSONField(
        verbose_name=_("Backend Kwargs"), blank=True, default=dict
    )
    message_kwargs = models.JSONField(
        verbose_name=_("Message Kwargs"), blank=True, default=dict
    )

    objects = models.Manager()

    def __str__(self):
        return str(self.name)

    class Meta:
        verbose_name = _("Message Template")
        db_table = "message_template"

    def mark_as_read(self):
        self.has_read = True
        self.save(update_fields=["has_read", "updated_at"])

    def mark_as_unread(self):
        self.has_read = False
        self.save(update_fields=["has_read", "updated_at"])

    def mark_as_sent(self):
        self.is_sent = True
        self.save(update_fields=["is_sent", "updated_at"])

    def mark_as_unsent(self):
        self.is_sent = False
        self.save(update_fields=["is_sent", "updated_at"])

    def mark_as_ignored(self):
        self.is_ignored = True
        self.save(update_fields=["is_ignored", "updated_at"])


class Message(models.Model):
    """
    Message
    """

    title = models.CharField(
        max_length=64, null=True, blank=True, verbose_name=_("Title")
    )
    mark = models.JSONField(
        verbose_name=_("Message Mark"),
        null=True,
        blank=True,
    )
    msg_type = models.CharField(
        max_length=64, verbose_name=_("Message Type"), db_index=True
    )
    content = models.TextField(verbose_name=_("Content"), null=True, blank=True)
    template = models.ForeignKey(
        MessageTemplate,
        verbose_name=_("Template"),
        db_constraint=False,
        null=True,
        blank=True,
        on_delete=models.CASCADE,
    )
    render_kwargs = models.JSONField(
        verbose_name=_("Render Kwargs"),
        blank=True,
        default=dict,
    )
    created_at = models.DateTimeField(auto_now_add=True, verbose_name=_("Created At"))

    objects = models.Manager()

    class Meta:
        verbose_name = _("Message")
        db_table = "message"
        indexes = [
            GinIndex(
                fields=["mark"],
                name="mark_gin_index",
            ),
        ]

    def __str__(self):
        return str(self.title)

    def save(self, **kwargs):
        if self.template:
            self.content = None

        super().save(**kwargs)




# Generated by Django 3.2.12 on 2022-08-08 07:36

import django.contrib.postgres.indexes
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('notification', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='message',
            name='mark',
            field=models.JSONField(blank=True, null=True, verbose_name='Message Mark'),
        ),
        migrations.AddIndex(
            model_name='message',
            index=django.contrib.postgres.indexes.GinIndex(fields=['mark'], name='mark_gin_index'),
        ),
    ]



from notification.backends.aliyunsms import (
    AliyunSMSNotificationBackend,
    notify_by_aliyun_sms,
)
from notification.backends.dingchatbot import (
    DingTalkChatbotNotificationBackend,
    notify_by_dingtalk_chatbot,
)
from notification.backends.dingtodotask import (
    DingTalkToDoTaskNotificationBackend,
    notify_by_dingtalk_todotask,
)
from notification.backends.dingworkmessage import (
    DingTalkWorkMessageNotificationBackend,
    notify_by_dingtalk_workmessage,
)
from notification.backends.dummy import DummyNotificationBackend, notify_by_dummy
from notification.backends.email import EmailNotificationBackend, notify_by_email
from notification.backends.websocket import (
    WebsocketNotificationBackend,
    notify_by_websocket,
)
from notification.base import notify

__all__ = [
    "notify",
    "notify_by_dummy",
    "notify_by_email",
    "notify_by_websocket",
    "notify_by_dingtalk_workmessage",
    "notify_by_dingtalk_chatbot",
    "notify_by_dingtalk_todotask",
    "notify_by_aliyun_sms",
    "DummyNotificationBackend",
    "EmailNotificationBackend",
    "WebsocketNotificationBackend",
    "DingTalkWorkMessageNotificationBackend",
    "DingTalkChatbotNotificationBackend",
    "DingTalkToDoTaskNotificationBackend",
    "AliyunSMSNotificationBackend",
]



import typing

import requests
from django.conf import settings
from django.contrib.auth.models import User
from django.core.exceptions import ImproperlyConfigured
from django.template import Context, Template
from markdownify import markdownify

from notification.base import BaseNotificationBackend, notify
from notification.models import Message, MessageTemplate


class DingTalkChatbotNotificationBackend(BaseNotificationBackend):
    """
    A backend handle dingtalk chatbot message.
    For details, see: https://ding-doc.dingtalk.com/doc#/serverapi2/qf2nxq
    """

    id = "dingtalkchatbot"
    message_subtype = "markdown"

    def __init__(self, *args, webhook=None, **kwargs):
        try:
            self.notification_settting = settings.DJANGO_USER_NOTIFICATION[self.id]
        except (AttributeError, KeyError):
            raise ImproperlyConfigured(
                "'DJANGO_USER_NOTIFICATION[{}]' must be set in settings.py".format(
                    self.id
                )
            )

        self.webhook = webhook or self.notification_settting.get("webhook")

        super().__init__(*args, **kwargs)

    def render_template(self, template: MessageTemplate, context: dict) -> str:
        try:
            html_content = Template(template.content.html).render(Context(context))
            return markdownify(html_content)
        except Exception as e:
            raise ValueError("Render message failed: %s" % e)

    def make_content(
        self, title, content, recipients, recipient_field, **kwargs
    ) -> dict:
        at_mobiles = kwargs.get("at_mobiles", [])
        if not at_mobiles and recipients and recipient_field:
            at_mobiles = [
                self.get_recipient(recipient, recipient_field)
                for recipient in recipients
            ]

        msgtype = kwargs.get("msgtype", self.message_subtype)
        body = {
            "msgtype": msgtype,
            msgtype: {"title": title, "text": content},
        }

        if kwargs.get("at_all"):
            body["at"] = {"isAtAll": True}
        elif at_mobiles:
            format_at_str = ("@" + str(mobile) for mobile in at_mobiles)
            content = " ".join(format_at_str) + "\n\n" + content
            body[msgtype]["text"] = content
            body["at"] = {"atMobiles": at_mobiles}

        return body

    def perform_send(
        self, message: Message, recipients, recipient_field, save, **kwargs
    ) -> None:
        """
        For details, see: https://ding-doc.dingtalk.com/doc#/serverapi2/qf2nxq
        """
        try:
            resp = requests.post(self.webhook, json=message.content)
            resp.raise_for_status()
            assert resp.json()["errcode"] == 0, resp.json()["errmsg"]
        except Exception as e:
            for recipient in recipients:
                self.on_failure(message, recipient, e, save=save)
        else:
            for recipient in recipients:
                self.on_success(message, recipient, save=save)


def notify_by_dingtalk_chatbot(
    recipients: list[User] = None,
    phone_field: typing.Union[str, typing.Callable] = None,
    title: str = None,
    message: str = None,
    context: dict = None,
    template_code: str = None,
    msgtype: str = "markdown",
    at_all: bool = False,
    at_mobiles: list[str] = None,
    webhook: str = None,
    save: bool = False,
    **kwargs,
):
    """
    Shortcut for dingtalk chatbot notification
    """
    message_kwargs = {
        "msgtype": msgtype,
        "at_all": at_all,
    }

    if at_mobiles:
        message_kwargs["at_mobiles"] = at_mobiles

    return notify(
        recipients or [],
        title=title,
        message=message,
        context=context,
        template_code=template_code,
        backends=(DingTalkChatbotNotificationBackend,),
        save=save,
        recipient_field=phone_field,
        message_kwargs=message_kwargs,
        webhook=webhook,
        **kwargs,
    )
